@@ -13,7 +13,7 @@ insert_header = function(doc) {
 ## Makes latex header with macros required for highlighting, tikz and framed
 make_header_latex = function() {
   h = "\\usepackage{graphicx, color}"
-  h = paste(c(h, opts_knit$get('header')), collapse = "\n")
+  h = paste(c(h, .header.maxwidth, opts_knit$get('header')), collapse = "\n")
   if (opts_knit$get('self.contained')) h else {
     writeLines(h, 'knitr.sty')
     '\\usepackage{knitr}'
@@ -25,11 +25,12 @@ insert_header_latex = function(doc, b) {
   if (length(i) == 1L) {
     l = str_locate(doc[i], b)
     tmp = str_sub(doc[i], l[, 1], l[, 2])
-    str_sub(doc[i], l[,1], l[,2]) = str_c(tmp, "\n", make_header_latex())
+    str_sub(doc[i], l[,1], l[,2]) = str_c(tmp, make_header_latex())
   } else if (length(i) == 0L && parent_mode()) {
     # in parent mode, we fill doc to be a complete document
-    doc = str_c(getOption('tikzDocumentDeclaration'), make_header_latex(),
-                .knitEnv$packages, "\\begin{document}", doc, "\\end{document}")
+    doc = str_c(c(getOption('tikzDocumentDeclaration'), make_header_latex(),
+                .knitEnv$tikzPackages, "\\begin{document}", doc, "\\end{document}"),
+                collapse = '\n')
   }
   doc
 }
@@ -107,10 +108,24 @@ set_header = function(...) {
 # CSS for html syntax highlighting
 .header.hi.html = paste(theme_to_header_html(.default.sty)$highlight,
                         collapse = '\n')
-rm(list = c('.default.sty', '.knitr.sty')) # do not need them any more
+rm(list = c('.inst.dir', '.default.sty', '.knitr.sty')) # do not need them any more
 
 .header.sweave.cmd =
 '\\newcommand{\\SweaveOpts}[1]{}  % do not interfere with LaTeX
 \\newcommand{\\SweaveInput}[1]{} % because they are not real TeX commands
 \\newcommand{\\Sexpr}[1]{}       % will only be parsed by R
 '
+
+.header.maxwidth =
+'%% maxwidth is the original width if it is less than linewidth
+%% otherwise use linewidth (to make sure the graphics do not exceed the margin)
+\\makeatletter
+\\def\\maxwidth{ %
+  \\ifdim\\Gin@nat@width>\\linewidth
+    \\linewidth
+  \\else
+    \\Gin@nat@width
+  \\fi
+}
+\\makeatother
+'
